#include "NWT_MOTORS.h"
#include "..\_Module\XMC_2Go_NWT.h"
#include "..\_Driver\GPIO.h"

uint8_t rMotorDirection = FORWARD, rMotorSpeed = 100;
uint8_t lMotorDirection = FORWARD, lMotorSpeed = 100;

uint8_t lWheelTick = 0, rWheelTick = 0;
int32_t lWheelDeg = 0, rWheelDeg = 0, robotDeg = 0;

void RMotor_init(void)
{
	RMotor_setMode(OUTPUT);
	RMotor_reset();
}

void RMotor_setDirection(uint8_t direction)
{
	rMotorDirection = direction;
}

void RMotor_setSpeed(uint8_t speed)
{
	rMotorSpeed = speed;
}

void RMotor_drive(void)
{	
	if(rMotorSpeed == 0)
		return;
	
	if(rMotorDirection == FORWARD)
	{
		RMotor_set();
		WAIT_uS(1500 - (500 * rMotorSpeed) / 100);
		RMotor_reset();
		WAIT_mS(3);
	}
	else if(rMotorDirection == BACKWARD)
	{
		RMotor_set();
		WAIT_uS(1500 + (500 * rMotorSpeed) / 100);
		RMotor_reset();
		WAIT_mS(3);
	}
}

void RMotor_brake(void)
{
	if(rMotorDirection == FORWARD)
	{
		RMotor_set();
		WAIT_uS(2000);
		RMotor_reset();
		WAIT_mS(3);
	}
	else if(rMotorDirection == BACKWARD)
	{
		RMotor_set();
		WAIT_uS(1000);
		RMotor_reset();
		WAIT_mS(3);
	}
}

void RMotor_coast(void)
{
	RMotor_reset();
}

void LMotor_init(void)
{
	LMotor_setMode(OUTPUT);
	LMotor_reset();
}

void LMotor_setDirection(uint8_t direction)
{
	lMotorDirection = direction;
}

void LMotor_setSpeed(uint8_t speed)
{
	lMotorSpeed = speed;
}

void LMotor_drive(void)
{
	if(lMotorSpeed == 0)
		return;
	
	if(lMotorDirection == FORWARD)
	{
		LMotor_set();
		WAIT_uS(1500 + (500 * lMotorSpeed) / 100);
		LMotor_reset();
		WAIT_mS(3);
	}
	else if(lMotorDirection == BACKWARD)
	{		
		LMotor_set();
		WAIT_uS(1500 - (500 * lMotorSpeed) / 100);
		LMotor_reset();
		WAIT_mS(3);
	}
}

void LMotor_brake(void)
{
	if(lMotorDirection == FORWARD)
	{
		LMotor_set();
		WAIT_uS(1000);
		LMotor_reset();
		WAIT_mS(3);
	}
	else if(lMotorDirection == BACKWARD)
	{
		LMotor_set();
		WAIT_uS(2000);
		LMotor_reset();
		WAIT_mS(3);
	}
}

void LMotor_coast(void)
{
	LMotor_reset();
}

void Odo_Init(void)
{
	LOdo_Init();
	ROdo_Init();
	
	lWheelDeg = 0;
	rWheelDeg = 0;
	robotDeg = 0;
}

void Odo_Update(void)
{
	if(LOdo_Read())
		lWheelTick = 1;
	else if(lWheelTick)
	{
		lWheelTick = 0;
		if(lMotorDirection == FORWARD)
			lWheelDeg += TICK_DEG;
		else
			lWheelDeg -= TICK_DEG;
	}
	
	if(ROdo_Read())
		rWheelTick = 1;
	else if(rWheelTick)
	{
		rWheelTick = 0;
		if(rMotorDirection == FORWARD)
			rWheelDeg += TICK_DEG;
		else
			rWheelDeg -= TICK_DEG;
	}
	
	robotDeg = (WHEEL_CIRC * (lWheelDeg + rWheelDeg)) / ROB_TURN_CIRC;
}

int32_t LOdo_getDeg(void)
{
	return lWheelDeg;
}

int32_t ROdo_getDeg(void)
{
	return rWheelDeg;
}

int32_t Odo_getRobDeg(void)
{
	return robotDeg;
}
